Package: isobar
Title: Analysis and quantitation of isobarically tagged MSMS proteomics
        data
Description: isobar provides methods for preprocessing, normalization, and
    report generation for the analysis of quantitative mass spectrometry
    proteomics data labeled with isobaric tags, such as iTRAQ and TMT.
    Features modules for integrating and validating PTM-centric datasets
    (isobar-PTM). More information on http://www.ms-isobar.org.
Version: 1.57.0
Author: Florian P Breitwieser <florian.bw@gmail.com> and
   Jacques Colinge <jacques.colinge@inserm.fr>, with contributions from 
   Alexey Stukalov <stukalov@biochem.mpg.de>,
   Xavier Robin <xavier.robin@unige.ch> and Florent Gluck  <florent.gluck@unige.ch>
Maintainer: Florian P Breitwieser <florian.bw@gmail.com>
biocViews: ImmunoOncology, Proteomics, MassSpectrometry,
        Bioinformatics, MultipleComparisons, QualityControl
Depends: R (>= 2.10.0), Biobase, stats, methods
Imports: distr, plyr, biomaRt, ggplot2
Suggests: MSnbase, OrgMassSpecR, XML, RJSONIO, Hmisc, gplots,
        RColorBrewer, gridExtra, limma, boot, DBI, MASS
LazyLoad: yes
License: LGPL-2
URL: https://github.com/fbreitwieser/isobar
BugReports: https://github.com/fbreitwieser/isobar/issues
Collate: utils.R ProteinGroup-class.R IBSpectra-class.R isobar-import.R
        IBSpectra-plots.R NoiseModel-class.R Tlsd-class.R
        ratio-methods.R distr-methods.R sharedpep-methods.R
        report-utils-xls.R report-utils-tex.R report-utils.R
        metareport-utils.R ptm-methods.R MSnSet-methods.R zzz.R
git_url: https://git.bioconductor.org/packages/isobar
git_branch: devel
git_last_commit: 0225060
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
NeedsCompilation: no
Packaged: 2025-10-31 22:27:40 UTC; biocbuild
